# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .task_retrieve_response import TaskRetrieveResponse as TaskRetrieveResponse
from .sound_effect_create_params import SoundEffectCreateParams as SoundEffectCreateParams
from .text_to_image_create_params import TextToImageCreateParams as TextToImageCreateParams
from .text_to_video_create_params import TextToVideoCreateParams as TextToVideoCreateParams
from .voice_dubbing_create_params import VoiceDubbingCreateParams as VoiceDubbingCreateParams
from .image_to_video_create_params import ImageToVideoCreateParams as ImageToVideoCreateParams
from .sound_effect_create_response import SoundEffectCreateResponse as SoundEffectCreateResponse
from .text_to_speech_create_params import TextToSpeechCreateParams as TextToSpeechCreateParams
from .video_to_video_create_params import VideoToVideoCreateParams as VideoToVideoCreateParams
from .text_to_image_create_response import TextToImageCreateResponse as TextToImageCreateResponse
from .text_to_video_create_response import TextToVideoCreateResponse as TextToVideoCreateResponse
from .voice_dubbing_create_response import VoiceDubbingCreateResponse as VoiceDubbingCreateResponse
from .voice_isolation_create_params import VoiceIsolationCreateParams as VoiceIsolationCreateParams
from .image_to_video_create_response import ImageToVideoCreateResponse as ImageToVideoCreateResponse
from .organization_retrieve_response import OrganizationRetrieveResponse as OrganizationRetrieveResponse
from .speech_to_speech_create_params import SpeechToSpeechCreateParams as SpeechToSpeechCreateParams
from .text_to_speech_create_response import TextToSpeechCreateResponse as TextToSpeechCreateResponse
from .video_to_video_create_response import VideoToVideoCreateResponse as VideoToVideoCreateResponse
from .voice_isolation_create_response import VoiceIsolationCreateResponse as VoiceIsolationCreateResponse
from .speech_to_speech_create_response import SpeechToSpeechCreateResponse as SpeechToSpeechCreateResponse
from .organization_retrieve_usage_params import OrganizationRetrieveUsageParams as OrganizationRetrieveUsageParams
from .character_performance_create_params import CharacterPerformanceCreateParams as CharacterPerformanceCreateParams
from .organization_retrieve_usage_response import OrganizationRetrieveUsageResponse as OrganizationRetrieveUsageResponse
from .character_performance_create_response import (
    CharacterPerformanceCreateResponse as CharacterPerformanceCreateResponse,
)



import time
import random
from typing import TYPE_CHECKING, Type, Union, TypeVar, Callable, Annotated, Coroutine, cast
from typing_extensions import ParamSpec, TypeAlias

import anyio

from runwayml._utils import PropertyInfo

from .._models import BaseModel
from ..types.task_retrieve_response import (
    Failed,
    Pending,
    Running,
    Cancelled,
    Succeeded,
    Throttled,
    TaskRetrieveResponse,
)

if TYPE_CHECKING:
    from .._client import RunwayML, AsyncRunwayML

P = ParamSpec("P")
T = TypeVar("T")

POLL_TIME = 6
POLL_JITTER = 3


class AwaitableTaskResponseMixin:
    def wait_for_task_output(self, timeout: Union[float, None] = 60 * 10) -> TaskRetrieveResponse:  # type: ignore[empty-body]
        """
        When called, this will block until the task is complete.

        If the task fails or is cancelled, a `TaskFailedError` will be raised.

        Args:
          timeout: The maximum amount of time to wait for the task to complete in seconds. If not
              specified, the default timeout is 10 minutes. Will raise a `TaskTimeoutError` if the
              task does not complete within the timeout.

        Returns:
          The task details, equivalent to calling `client.tasks.retrieve(task_id)`.
        """
        ...


class NewTaskCreatedResponse(AwaitableTaskResponseMixin, BaseModel):
    id: str


class AsyncAwaitableTaskResponseMixin:
    async def wait_for_task_output(self, timeout: Union[float, None] = 60 * 10) -> TaskRetrieveResponse:  # type: ignore[empty-body]
        """
        When called, this will wait until the task is complete.

        If the task fails or is cancelled, a `TaskFailedError` will be raised.

        Args:
          timeout: The maximum amount of time to wait for the task to complete in seconds. If not
              specified, the default timeout is 10 minutes. Will raise a `TaskTimeoutError` if the
              task does not complete within the timeout. Setting this to `None` will wait
              indefinitely (disabling the timeout).

        Returns:
          The task details, equivalent to awaiting `client.tasks.retrieve(task_id)`.
        """
        ...


class AsyncNewTaskCreatedResponse(AsyncAwaitableTaskResponseMixin, BaseModel):
    id: str


def create_waitable_resource(base_class: Type[T], client: "RunwayML") -> Type[NewTaskCreatedResponse]:
    class WithClient(base_class):  # type: ignore[valid-type,misc]
        id: str

        def wait_for_task_output(self, timeout: Union[float, None] = 60 * 10) -> TaskRetrieveResponse:
            start_time = time.time()
            while True:
                time.sleep(POLL_TIME + random.random() * POLL_JITTER - POLL_JITTER / 2)
                task_details = client.tasks.retrieve(self.id)
                if task_details.status == "SUCCEEDED":
                    return task_details
                if task_details.status == "FAILED":
                    raise TaskFailedError(task_details)
                if timeout is not None and time.time() - start_time > timeout:
                    raise TaskTimeoutError(task_details)

    WithClient.__name__ = base_class.__name__
    WithClient.__qualname__ = base_class.__qualname__

    return cast(Type[NewTaskCreatedResponse], WithClient)


def create_async_waitable_resource(base_class: Type[T], client: "AsyncRunwayML") -> Type[AsyncNewTaskCreatedResponse]:
    class WithClient(base_class):  # type: ignore[valid-type,misc]
        id: str

        async def wait_for_task_output(self, timeout: Union[float, None] = 60 * 10) -> TaskRetrieveResponse:
            start_time = anyio.current_time()
            while True:
                await anyio.sleep(POLL_TIME + random.random() * POLL_JITTER - POLL_JITTER / 2)
                task_details = await client.tasks.retrieve(self.id)
                if task_details.status == "SUCCEEDED":
                    return task_details
                if task_details.status == "FAILED" or task_details.status == "CANCELLED":
                    raise TaskFailedError(task_details)
                if timeout is not None and anyio.current_time() - start_time > timeout:
                    raise TaskTimeoutError(task_details)

    WithClient.__name__ = base_class.__name__
    WithClient.__qualname__ = base_class.__qualname__

    return cast(Type[AsyncNewTaskCreatedResponse], WithClient)


class TaskFailedError(Exception):
    def __init__(self, task_details: TaskRetrieveResponse):
        self.task_details = task_details
        super().__init__(f"Task failed")


class TaskTimeoutError(Exception):
    def __init__(self, task_details: TaskRetrieveResponse):
        self.task_details = task_details
        super().__init__(f"Task timed out")


class AwaitablePending(AwaitableTaskResponseMixin, Pending): ...


class AwaitableThrottled(AwaitableTaskResponseMixin, Throttled): ...


class AwaitableCancelled(AwaitableTaskResponseMixin, Cancelled): ...


class AwaitableRunning(AwaitableTaskResponseMixin, Running): ...


class AwaitableFailed(AwaitableTaskResponseMixin, Failed): ...


class AwaitableSucceeded(AwaitableTaskResponseMixin, Succeeded): ...


AwaitableTaskRetrieveResponse: TypeAlias = Annotated[
    Union[
        AwaitablePending, AwaitableThrottled, AwaitableCancelled, AwaitableRunning, AwaitableFailed, AwaitableSucceeded
    ],
    PropertyInfo(discriminator="status"),
]


class AsyncAwaitablePending(AsyncAwaitableTaskResponseMixin, Pending): ...


class AsyncAwaitableThrottled(AsyncAwaitableTaskResponseMixin, Throttled): ...


class AsyncAwaitableCancelled(AsyncAwaitableTaskResponseMixin, Cancelled): ...


class AsyncAwaitableRunning(AsyncAwaitableTaskResponseMixin, Running): ...


class AsyncAwaitableFailed(AsyncAwaitableTaskResponseMixin, Failed): ...


class AsyncAwaitableSucceeded(AsyncAwaitableTaskResponseMixin, Succeeded): ...


AsyncAwaitableTaskRetrieveResponse: TypeAlias = Annotated[
    Union[
        AsyncAwaitablePending,
        AsyncAwaitableThrottled,
        AsyncAwaitableCancelled,
        AsyncAwaitableRunning,
        AsyncAwaitableFailed,
        AsyncAwaitableSucceeded,
    ],
    PropertyInfo(discriminator="status"),
]


def _make_sync_wait_for_task_output(
    client: "RunwayML",
) -> Callable[["AwaitableTaskResponseMixin", Union[float, None]], TaskRetrieveResponse]:
    """Create a wait_for_task_output method bound to the given client."""

    def wait_for_task_output(
        self: "AwaitableTaskResponseMixin", timeout: Union[float, None] = 60 * 10
    ) -> TaskRetrieveResponse:
        start_time = time.time()
        while True:
            time.sleep(POLL_TIME + random.random() * POLL_JITTER - POLL_JITTER / 2)
            task_details = client.tasks.retrieve(self.id)  # type: ignore[attr-defined]
            if task_details.status == "SUCCEEDED":
                return task_details
            if task_details.status == "FAILED":
                raise TaskFailedError(task_details)
            if timeout is not None and time.time() - start_time > timeout:
                raise TaskTimeoutError(task_details)

    return wait_for_task_output


def inject_sync_wait_method(client: "RunwayML", response: T) -> T:
    """Inject the wait_for_task_output method onto the response instance."""
    import types

    response.wait_for_task_output = types.MethodType(_make_sync_wait_for_task_output(client), response)  # type: ignore[attr-defined]
    return response


def _make_async_wait_for_task_output(
    client: "AsyncRunwayML",
) -> Callable[["AsyncAwaitableTaskResponseMixin", Union[float, None]], Coroutine[None, None, TaskRetrieveResponse]]:
    """Create an async wait_for_task_output method bound to the given client."""

    async def wait_for_task_output(
        self: "AsyncAwaitableTaskResponseMixin", timeout: Union[float, None] = 60 * 10
    ) -> TaskRetrieveResponse:
        start_time = anyio.current_time()
        while True:
            await anyio.sleep(POLL_TIME + random.random() * POLL_JITTER - POLL_JITTER / 2)
            task_details = await client.tasks.retrieve(self.id)  # type: ignore[attr-defined]
            if task_details.status == "SUCCEEDED":
                return task_details
            if task_details.status == "FAILED" or task_details.status == "CANCELLED":
                raise TaskFailedError(task_details)
            if timeout is not None and anyio.current_time() - start_time > timeout:
                raise TaskTimeoutError(task_details)

    return wait_for_task_output


def inject_async_wait_method(client: "AsyncRunwayML", response: T) -> T:
    """Inject the async wait_for_task_output method onto the response instance."""
    import types

    response.wait_for_task_output = types.MethodType(_make_async_wait_for_task_output(client), response)  # type: ignore[attr-defined]
    return response



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .._models import BaseModel

__all__ = ["UploadCreateEphemeralResponse"]


class UploadCreateEphemeralResponse(BaseModel):
    uri: str
    """The Runway upload URI to use in other API generation requests."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable
from typing_extensions import Literal, Required, Annotated, TypeAlias, TypedDict

from .._utils import PropertyInfo

__all__ = [
    "ImageToVideoCreateParams",
    "Gen4_5",
    "Gen4_5PromptImagePromptImage",
    "Gen4_5ContentModeration",
    "Gen4Turbo",
    "Gen4TurboPromptImagePromptImage",
    "Gen4TurboContentModeration",
    "Veo3_1",
    "Veo3_1PromptImagePromptImage",
    "Gen3aTurbo",
    "Gen3aTurboPromptImagePromptImage",
    "Gen3aTurboContentModeration",
    "Veo3_1Fast",
    "Veo3_1FastPromptImagePromptImage",
    "Veo3",
    "Veo3PromptImagePromptImage",
]


class Gen4_5(TypedDict, total=False):
    duration: Required[int]
    """The number of seconds of duration for the output video.

    Must be an integer from 2 to 10.
    """

    model: Required[Literal["gen4.5"]]

    prompt_image: Required[
        Annotated[Union[str, Iterable[Gen4_5PromptImagePromptImage]], PropertyInfo(alias="promptImage")]
    ]
    """A HTTPS URL."""

    prompt_text: Required[Annotated[str, PropertyInfo(alias="promptText")]]
    """A non-empty string up to 1000 characters (measured in UTF-16 code units).

    This should describe in detail what should appear in the output.
    """

    ratio: Required[Literal["1280:720", "720:1280", "1104:832", "960:960", "832:1104", "1584:672"]]
    """The resolution of the output video."""

    content_moderation: Annotated[Gen4_5ContentModeration, PropertyInfo(alias="contentModeration")]
    """Settings that affect the behavior of the content moderation system."""

    seed: int
    """If unspecified, a random number is chosen.

    Varying the seed integer is a way to get different results for the same other
    request parameters. Using the same seed integer for an identical request will
    produce similar results.
    """


class Gen4_5PromptImagePromptImage(TypedDict, total=False):
    position: Required[Literal["first"]]
    """The position of the image in the output video.

    "first" will use the image as the first frame of the video.
    """

    uri: Required[str]
    """A HTTPS URL."""


class Gen4_5ContentModeration(TypedDict, total=False):
    """Settings that affect the behavior of the content moderation system."""

    public_figure_threshold: Annotated[Literal["auto", "low"], PropertyInfo(alias="publicFigureThreshold")]
    """
    When set to `low`, the content moderation system will be less strict about
    preventing generations that include recognizable public figures.
    """


class Gen4Turbo(TypedDict, total=False):
    model: Required[Literal["gen4_turbo"]]

    prompt_image: Required[
        Annotated[Union[str, Iterable[Gen4TurboPromptImagePromptImage]], PropertyInfo(alias="promptImage")]
    ]
    """A HTTPS URL."""

    ratio: Required[Literal["1280:720", "720:1280", "1104:832", "832:1104", "960:960", "1584:672"]]
    """The resolution of the output video."""

    content_moderation: Annotated[Gen4TurboContentModeration, PropertyInfo(alias="contentModeration")]
    """Settings that affect the behavior of the content moderation system."""

    duration: int
    """The number of seconds of duration for the output video."""

    prompt_text: Annotated[str, PropertyInfo(alias="promptText")]
    """A non-empty string up to 1000 characters (measured in UTF-16 code units).

    This should describe in detail what should appear in the output.
    """

    seed: int
    """If unspecified, a random number is chosen.

    Varying the seed integer is a way to get different results for the same other
    request parameters. Using the same seed integer for an identical request will
    produce similar results.
    """


class Gen4TurboPromptImagePromptImage(TypedDict, total=False):
    position: Required[Literal["first"]]
    """The position of the image in the output video.

    "first" will use the image as the first frame of the video.
    """

    uri: Required[str]
    """A HTTPS URL."""


class Gen4TurboContentModeration(TypedDict, total=False):
    """Settings that affect the behavior of the content moderation system."""

    public_figure_threshold: Annotated[Literal["auto", "low"], PropertyInfo(alias="publicFigureThreshold")]
    """
    When set to `low`, the content moderation system will be less strict about
    preventing generations that include recognizable public figures.
    """


class Veo3_1(TypedDict, total=False):
    model: Required[Literal["veo3.1"]]

    prompt_image: Required[
        Annotated[Union[str, Iterable[Veo3_1PromptImagePromptImage]], PropertyInfo(alias="promptImage")]
    ]
    """
    You may specify an image to use as the first frame of the output video, or an
    array with a first frame and optionally a last frame. This model does not
    support generating with only a last frame.
    """

    ratio: Required[Literal["1280:720", "720:1280", "1080:1920", "1920:1080"]]
    """The resolution of the output video."""

    audio: bool
    """Whether to generate audio for the video. Audio inclusion affects pricing."""

    duration: Literal[4, 6, 8]
    """The number of seconds of duration for the output video."""

    prompt_text: Annotated[str, PropertyInfo(alias="promptText")]
    """A non-empty string up to 1000 characters (measured in UTF-16 code units).

    This should describe in detail what should appear in the output.
    """


class Veo3_1PromptImagePromptImage(TypedDict, total=False):
    position: Required[Literal["first", "last"]]
    """The position of the image in the output video.

    "first" will use the image as the first frame of the video, "last" will use the
    image as the last frame of the video.
    """

    uri: Required[str]
    """A HTTPS URL."""


class Gen3aTurbo(TypedDict, total=False):
    model: Required[Literal["gen3a_turbo"]]

    prompt_image: Required[
        Annotated[Union[str, Iterable[Gen3aTurboPromptImagePromptImage]], PropertyInfo(alias="promptImage")]
    ]
    """A HTTPS URL."""

    prompt_text: Required[Annotated[str, PropertyInfo(alias="promptText")]]
    """A non-empty string up to 1000 characters (measured in UTF-16 code units).

    This should describe in detail what should appear in the output.
    """

    content_moderation: Annotated[Gen3aTurboContentModeration, PropertyInfo(alias="contentModeration")]
    """Settings that affect the behavior of the content moderation system."""

    duration: Literal[5, 10]
    """The duration of the output video in seconds."""

    ratio: Literal["768:1280", "1280:768"]
    """The resolution of the output video."""

    seed: int
    """If unspecified, a random number is chosen.

    Varying the seed integer is a way to get different results for the same other
    request parameters. Using the same seed integer for an identical request will
    produce similar results.
    """


class Gen3aTurboPromptImagePromptImage(TypedDict, total=False):
    position: Required[Literal["first", "last"]]
    """The position of the image in the output video.

    "first" will use the image as the first frame of the video, "last" will use the
    image as the last frame of the video.
    """

    uri: Required[str]
    """A HTTPS URL."""


class Gen3aTurboContentModeration(TypedDict, total=False):
    """Settings that affect the behavior of the content moderation system."""

    public_figure_threshold: Annotated[Literal["auto", "low"], PropertyInfo(alias="publicFigureThreshold")]
    """
    When set to `low`, the content moderation system will be less strict about
    preventing generations that include recognizable public figures.
    """


class Veo3_1Fast(TypedDict, total=False):
    model: Required[Literal["veo3.1_fast"]]

    prompt_image: Required[
        Annotated[Union[str, Iterable[Veo3_1FastPromptImagePromptImage]], PropertyInfo(alias="promptImage")]
    ]
    """
    You may specify an image to use as the first frame of the output video, or an
    array with a first frame and optionally a last frame. This model does not
    support generating with only a last frame.
    """

    ratio: Required[Literal["1280:720", "720:1280", "1080:1920", "1920:1080"]]
    """The resolution of the output video."""

    audio: bool
    """Whether to generate audio for the video. Audio inclusion affects pricing."""

    duration: Literal[4, 6, 8]
    """The number of seconds of duration for the output video."""

    prompt_text: Annotated[str, PropertyInfo(alias="promptText")]
    """A non-empty string up to 1000 characters (measured in UTF-16 code units).

    This should describe in detail what should appear in the output.
    """


class Veo3_1FastPromptImagePromptImage(TypedDict, total=False):
    position: Required[Literal["first", "last"]]
    """The position of the image in the output video.

    "first" will use the image as the first frame of the video, "last" will use the
    image as the last frame of the video.
    """

    uri: Required[str]
    """A HTTPS URL."""


class Veo3(TypedDict, total=False):
    duration: Required[Literal[8]]
    """The number of seconds of duration for the output video."""

    model: Required[Literal["veo3"]]

    prompt_image: Required[
        Annotated[Union[str, Iterable[Veo3PromptImagePromptImage]], PropertyInfo(alias="promptImage")]
    ]
    """A HTTPS URL."""

    ratio: Required[Literal["1280:720", "720:1280", "1080:1920", "1920:1080"]]
    """The resolution of the output video."""

    prompt_text: Annotated[str, PropertyInfo(alias="promptText")]
    """A non-empty string up to 1000 characters (measured in UTF-16 code units).

    This should describe in detail what should appear in the output.
    """


class Veo3PromptImagePromptImage(TypedDict, total=False):
    position: Required[Literal["first"]]
    """The position of the image in the output video.

    "first" will use the image as the first frame of the video.
    """

    uri: Required[str]
    """A HTTPS URL."""


ImageToVideoCreateParams: TypeAlias = Union[Gen4_5, Gen4Turbo, Veo3_1, Gen3aTurbo, Veo3_1Fast, Veo3]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from runwayml import RunwayML, AsyncRunwayML
from tests.utils import assert_matches_type
from runwayml.types import ImageToVideoCreateResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestImageToVideo:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create_overload_1(self, client: RunwayML) -> None:
        image_to_video = client.image_to_video.create(
            duration=2,
            model="gen4.5",
            prompt_image="https://example.com/file",
            prompt_text="x",
            ratio="1280:720",
        )
        assert_matches_type(ImageToVideoCreateResponse, image_to_video, path=["response"])

    @parametrize
    def test_method_create_with_all_params_overload_1(self, client: RunwayML) -> None:
        image_to_video = client.image_to_video.create(
            duration=2,
            model="gen4.5",
            prompt_image="https://example.com/file",
            prompt_text="x",
            ratio="1280:720",
            content_moderation={"public_figure_threshold": "auto"},
            seed=0,
        )
        assert_matches_type(ImageToVideoCreateResponse, image_to_video, path=["response"])

    @parametrize
    def test_raw_response_create_overload_1(self, client: RunwayML) -> None:
        response = client.image_to_video.with_raw_response.create(
            duration=2,
            model="gen4.5",
            prompt_image="https://example.com/file",
            prompt_text="x",
            ratio="1280:720",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        image_to_video = response.parse()
        assert_matches_type(ImageToVideoCreateResponse, image_to_video, path=["response"])

    @parametrize
    def test_streaming_response_create_overload_1(self, client: RunwayML) -> None:
        with client.image_to_video.with_streaming_response.create(
            duration=2,
            model="gen4.5",
            prompt_image="https://example.com/file",
            prompt_text="x",
            ratio="1280:720",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            image_to_video = response.parse()
            assert_matches_type(ImageToVideoCreateResponse, image_to_video, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_create_overload_2(self, client: RunwayML) -> None:
        image_to_video = client.image_to_video.create(
            model="gen4_turbo",
            prompt_image="https://example.com/file",
            ratio="1280:720",
        )
        assert_matches_type(ImageToVideoCreateResponse, image_to_video, path=["response"])

    @parametrize
    def test_method_create_with_all_params_overload_2(self, client: RunwayML) -> None:
        image_to_video = client.image_to_video.create(
            model="gen4_turbo",
            prompt_image="https://example.com/file",
            ratio="1280:720",
            content_moderation={"public_figure_threshold": "auto"},
            duration=2,
            prompt_text="x",
            seed=0,
        )
        assert_matches_type(ImageToVideoCreateResponse, image_to_video, path=["response"])

    @parametrize
    def test_raw_response_create_overload_2(self, client: RunwayML) -> None:
        response = client.image_to_video.with_raw_response.create(
            model="gen4_turbo",
            prompt_image="https://example.com/file",
            ratio="1280:720",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        image_to_video = response.parse()
        assert_matches_type(ImageToVideoCreateResponse, image_to_video, path=["response"])

    @parametrize
    def test_streaming_response_create_overload_2(self, client: RunwayML) -> None:
        with client.image_to_video.with_streaming_response.create(
            model="gen4_turbo",
            prompt_image="https://example.com/file",
            ratio="1280:720",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            image_to_video = response.parse()
            assert_matches_type(ImageToVideoCreateResponse, image_to_video, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_create_overload_3(self, client: RunwayML) -> None:
        image_to_video = client.image_to_video.create(
            model="veo3.1",
            prompt_image="https://example.com/file",
            ratio="1280:720",
        )
        assert_matches_type(ImageToVideoCreateResponse, image_to_video, path=["response"])

    @parametrize
    def test_method_create_with_all_params_overload_3(self, client: RunwayML) -> None:
        image_to_video = client.image_to_video.create(
            model="veo3.1",
            prompt_image="https://example.com/file",
            ratio="1280:720",
            audio=True,
            duration=4,
            prompt_text="x",
        )
        assert_matches_type(ImageToVideoCreateResponse, image_to_video, path=["response"])

    @parametrize
    def test_raw_response_create_overload_3(self, client: RunwayML) -> None:
        response = client.image_to_video.with_raw_response.create(
            model="veo3.1",
            prompt_image="https://example.com/file",
            ratio="1280:720",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        image_to_video = response.parse()
        assert_matches_type(ImageToVideoCreateResponse, image_to_video, path=["response"])

    @parametrize
    def test_streaming_response_create_overload_3(self, client: RunwayML) -> None:
        with client.image_to_video.with_streaming_response.create(
            model="veo3.1",
            prompt_image="https://example.com/file",
            ratio="1280:720",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            image_to_video = response.parse()
            assert_matches_type(ImageToVideoCreateResponse, image_to_video, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_create_overload_4(self, client: RunwayML) -> None:
        image_to_video = client.image_to_video.create(
            model="gen3a_turbo",
            prompt_image="https://example.com/file",
            prompt_text="A beautiful sunset over a calm ocean.",
        )
        assert_matches_type(ImageToVideoCreateResponse, image_to_video, path=["response"])

    @parametrize
    def test_method_create_with_all_params_overload_4(self, client: RunwayML) -> None:
        image_to_video = client.image_to_video.create(
            model="gen3a_turbo",
            prompt_image="https://example.com/file",
            prompt_text="A beautiful sunset over a calm ocean.",
            content_moderation={"public_figure_threshold": "auto"},
            duration=5,
            ratio="1280:768",
            seed=0,
        )
        assert_matches_type(ImageToVideoCreateResponse, image_to_video, path=["response"])

    @parametrize
    def test_raw_response_create_overload_4(self, client: RunwayML) -> None:
        response = client.image_to_video.with_raw_response.create(
            model="gen3a_turbo",
            prompt_image="https://example.com/file",
            prompt_text="A beautiful sunset over a calm ocean.",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        image_to_video = response.parse()
        assert_matches_type(ImageToVideoCreateResponse, image_to_video, path=["response"])

    @parametrize
    def test_streaming_response_create_overload_4(self, client: RunwayML) -> None:
        with client.image_to_video.with_streaming_response.create(
            model="gen3a_turbo",
            prompt_image="https://example.com/file",
            prompt_text="A beautiful sunset over a calm ocean.",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            image_to_video = response.parse()
            assert_matches_type(ImageToVideoCreateResponse, image_to_video, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_create_overload_5(self, client: RunwayML) -> None:
        image_to_video = client.image_to_video.create(
            model="veo3.1_fast",
            prompt_image="https://example.com/file",
            ratio="1280:720",
        )
        assert_matches_type(ImageToVideoCreateResponse, image_to_video, path=["response"])

    @parametrize
    def test_method_create_with_all_params_overload_5(self, client: RunwayML) -> None:
        image_to_video = client.image_to_video.create(
            model="veo3.1_fast",
            prompt_image="https://example.com/file",
            ratio="1280:720",
            audio=True,
            duration=4,
            prompt_text="x",
        )
        assert_matches_type(ImageToVideoCreateResponse, image_to_video, path=["response"])

    @parametrize
    def test_raw_response_create_overload_5(self, client: RunwayML) -> None:
        response = client.image_to_video.with_raw_response.create(
            model="veo3.1_fast",
            prompt_image="https://example.com/file",
            ratio="1280:720",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        image_to_video = response.parse()
        assert_matches_type(ImageToVideoCreateResponse, image_to_video, path=["response"])

    @parametrize
    def test_streaming_response_create_overload_5(self, client: RunwayML) -> None:
        with client.image_to_video.with_streaming_response.create(
            model="veo3.1_fast",
            prompt_image="https://example.com/file",
            ratio="1280:720",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            image_to_video = response.parse()
            assert_matches_type(ImageToVideoCreateResponse, image_to_video, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_create_overload_6(self, client: RunwayML) -> None:
        image_to_video = client.image_to_video.create(
            duration=8,
            model="veo3",
            prompt_image="https://example.com/file",
            ratio="1280:720",
        )
        assert_matches_type(ImageToVideoCreateResponse, image_to_video, path=["response"])

    @parametrize
    def test_method_create_with_all_params_overload_6(self, client: RunwayML) -> None:
        image_to_video = client.image_to_video.create(
            duration=8,
            model="veo3",
            prompt_image="https://example.com/file",
            ratio="1280:720",
            prompt_text="x",
        )
        assert_matches_type(ImageToVideoCreateResponse, image_to_video, path=["response"])

    @parametrize
    def test_raw_response_create_overload_6(self, client: RunwayML) -> None:
        response = client.image_to_video.with_raw_response.create(
            duration=8,
            model="veo3",
            prompt_image="https://example.com/file",
            ratio="1280:720",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        image_to_video = response.parse()
        assert_matches_type(ImageToVideoCreateResponse, image_to_video, path=["response"])

    @parametrize
    def test_streaming_response_create_overload_6(self, client: RunwayML) -> None:
        with client.image_to_video.with_streaming_response.create(
            duration=8,
            model="veo3",
            prompt_image="https://example.com/file",
            ratio="1280:720",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            image_to_video = response.parse()
            assert_matches_type(ImageToVideoCreateResponse, image_to_video, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncImageToVideo:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_create_overload_1(self, async_client: AsyncRunwayML) -> None:
        image_to_video = await async_client.image_to_video.create(
            duration=2,
            model="gen4.5",
            prompt_image="https://example.com/file",
            prompt_text="x",
            ratio="1280:720",
        )
        assert_matches_type(ImageToVideoCreateResponse, image_to_video, path=["response"])

    @parametrize
    async def test_method_create_with_all_params_overload_1(self, async_client: AsyncRunwayML) -> None:
        image_to_video = await async_client.image_to_video.create(
            duration=2,
            model="gen4.5",
            prompt_image="https://example.com/file",
            prompt_text="x",
            ratio="1280:720",
            content_moderation={"public_figure_threshold": "auto"},
            seed=0,
        )
        assert_matches_type(ImageToVideoCreateResponse, image_to_video, path=["response"])

    @parametrize
    async def test_raw_response_create_overload_1(self, async_client: AsyncRunwayML) -> None:
        response = await async_client.image_to_video.with_raw_response.create(
            duration=2,
            model="gen4.5",
            prompt_image="https://example.com/file",
            prompt_text="x",
            ratio="1280:720",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        image_to_video = await response.parse()
        assert_matches_type(ImageToVideoCreateResponse, image_to_video, path=["response"])

    @parametrize
    async def test_streaming_response_create_overload_1(self, async_client: AsyncRunwayML) -> None:
        async with async_client.image_to_video.with_streaming_response.create(
            duration=2,
            model="gen4.5",
            prompt_image="https://example.com/file",
            prompt_text="x",
            ratio="1280:720",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            image_to_video = await response.parse()
            assert_matches_type(ImageToVideoCreateResponse, image_to_video, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_create_overload_2(self, async_client: AsyncRunwayML) -> None:
        image_to_video = await async_client.image_to_video.create(
            model="gen4_turbo",
            prompt_image="https://example.com/file",
            ratio="1280:720",
        )
        assert_matches_type(ImageToVideoCreateResponse, image_to_video, path=["response"])

    @parametrize
    async def test_method_create_with_all_params_overload_2(self, async_client: AsyncRunwayML) -> None:
        image_to_video = await async_client.image_to_video.create(
            model="gen4_turbo",
            prompt_image="https://example.com/file",
            ratio="1280:720",
            content_moderation={"public_figure_threshold": "auto"},
            duration=2,
            prompt_text="x",
            seed=0,
        )
        assert_matches_type(ImageToVideoCreateResponse, image_to_video, path=["response"])

    @parametrize
    async def test_raw_response_create_overload_2(self, async_client: AsyncRunwayML) -> None:
        response = await async_client.image_to_video.with_raw_response.create(
            model="gen4_turbo",
            prompt_image="https://example.com/file",
            ratio="1280:720",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        image_to_video = await response.parse()
        assert_matches_type(ImageToVideoCreateResponse, image_to_video, path=["response"])

    @parametrize
    async def test_streaming_response_create_overload_2(self, async_client: AsyncRunwayML) -> None:
        async with async_client.image_to_video.with_streaming_response.create(
            model="gen4_turbo",
            prompt_image="https://example.com/file",
            ratio="1280:720",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            image_to_video = await response.parse()
            assert_matches_type(ImageToVideoCreateResponse, image_to_video, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_create_overload_3(self, async_client: AsyncRunwayML) -> None:
        image_to_video = await async_client.image_to_video.create(
            model="veo3.1",
            prompt_image="https://example.com/file",
            ratio="1280:720",
        )
        assert_matches_type(ImageToVideoCreateResponse, image_to_video, path=["response"])

    @parametrize
    async def test_method_create_with_all_params_overload_3(self, async_client: AsyncRunwayML) -> None:
        image_to_video = await async_client.image_to_video.create(
            model="veo3.1",
            prompt_image="https://example.com/file",
            ratio="1280:720",
            audio=True,
            duration=4,
            prompt_text="x",
        )
        assert_matches_type(ImageToVideoCreateResponse, image_to_video, path=["response"])

    @parametrize
    async def test_raw_response_create_overload_3(self, async_client: AsyncRunwayML) -> None:
        response = await async_client.image_to_video.with_raw_response.create(
            model="veo3.1",
            prompt_image="https://example.com/file",
            ratio="1280:720",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        image_to_video = await response.parse()
        assert_matches_type(ImageToVideoCreateResponse, image_to_video, path=["response"])

    @parametrize
    async def test_streaming_response_create_overload_3(self, async_client: AsyncRunwayML) -> None:
        async with async_client.image_to_video.with_streaming_response.create(
            model="veo3.1",
            prompt_image="https://example.com/file",
            ratio="1280:720",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            image_to_video = await response.parse()
            assert_matches_type(ImageToVideoCreateResponse, image_to_video, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_create_overload_4(self, async_client: AsyncRunwayML) -> None:
        image_to_video = await async_client.image_to_video.create(
            model="gen3a_turbo",
            prompt_image="https://example.com/file",
            prompt_text="A beautiful sunset over a calm ocean.",
        )
        assert_matches_type(ImageToVideoCreateResponse, image_to_video, path=["response"])

    @parametrize
    async def test_method_create_with_all_params_overload_4(self, async_client: AsyncRunwayML) -> None:
        image_to_video = await async_client.image_to_video.create(
            model="gen3a_turbo",
            prompt_image="https://example.com/file",
            prompt_text="A beautiful sunset over a calm ocean.",
            content_moderation={"public_figure_threshold": "auto"},
            duration=5,
            ratio="1280:768",
            seed=0,
        )
        assert_matches_type(ImageToVideoCreateResponse, image_to_video, path=["response"])

    @parametrize
    async def test_raw_response_create_overload_4(self, async_client: AsyncRunwayML) -> None:
        response = await async_client.image_to_video.with_raw_response.create(
            model="gen3a_turbo",
            prompt_image="https://example.com/file",
            prompt_text="A beautiful sunset over a calm ocean.",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        image_to_video = await response.parse()
        assert_matches_type(ImageToVideoCreateResponse, image_to_video, path=["response"])

    @parametrize
    async def test_streaming_response_create_overload_4(self, async_client: AsyncRunwayML) -> None:
        async with async_client.image_to_video.with_streaming_response.create(
            model="gen3a_turbo",
            prompt_image="https://example.com/file",
            prompt_text="A beautiful sunset over a calm ocean.",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            image_to_video = await response.parse()
            assert_matches_type(ImageToVideoCreateResponse, image_to_video, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_create_overload_5(self, async_client: AsyncRunwayML) -> None:
        image_to_video = await async_client.image_to_video.create(
            model="veo3.1_fast",
            prompt_image="https://example.com/file",
            ratio="1280:720",
        )
        assert_matches_type(ImageToVideoCreateResponse, image_to_video, path=["response"])

    @parametrize
    async def test_method_create_with_all_params_overload_5(self, async_client: AsyncRunwayML) -> None:
        image_to_video = await async_client.image_to_video.create(
            model="veo3.1_fast",
            prompt_image="https://example.com/file",
            ratio="1280:720",
            audio=True,
            duration=4,
            prompt_text="x",
        )
        assert_matches_type(ImageToVideoCreateResponse, image_to_video, path=["response"])

    @parametrize
    async def test_raw_response_create_overload_5(self, async_client: AsyncRunwayML) -> None:
        response = await async_client.image_to_video.with_raw_response.create(
            model="veo3.1_fast",
            prompt_image="https://example.com/file",
            ratio="1280:720",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        image_to_video = await response.parse()
        assert_matches_type(ImageToVideoCreateResponse, image_to_video, path=["response"])

    @parametrize
    async def test_streaming_response_create_overload_5(self, async_client: AsyncRunwayML) -> None:
        async with async_client.image_to_video.with_streaming_response.create(
            model="veo3.1_fast",
            prompt_image="https://example.com/file",
            ratio="1280:720",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            image_to_video = await response.parse()
            assert_matches_type(ImageToVideoCreateResponse, image_to_video, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_create_overload_6(self, async_client: AsyncRunwayML) -> None:
        image_to_video = await async_client.image_to_video.create(
            duration=8,
            model="veo3",
            prompt_image="https://example.com/file",
            ratio="1280:720",
        )
        assert_matches_type(ImageToVideoCreateResponse, image_to_video, path=["response"])

    @parametrize
    async def test_method_create_with_all_params_overload_6(self, async_client: AsyncRunwayML) -> None:
        image_to_video = await async_client.image_to_video.create(
            duration=8,
            model="veo3",
            prompt_image="https://example.com/file",
            ratio="1280:720",
            prompt_text="x",
        )
        assert_matches_type(ImageToVideoCreateResponse, image_to_video, path=["response"])

    @parametrize
    async def test_raw_response_create_overload_6(self, async_client: AsyncRunwayML) -> None:
        response = await async_client.image_to_video.with_raw_response.create(
            duration=8,
            model="veo3",
            prompt_image="https://example.com/file",
            ratio="1280:720",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        image_to_video = await response.parse()
        assert_matches_type(ImageToVideoCreateResponse, image_to_video, path=["response"])

    @parametrize
    async def test_streaming_response_create_overload_6(self, async_client: AsyncRunwayML) -> None:
        async with async_client.image_to_video.with_streaming_response.create(
            duration=8,
            model="veo3",
            prompt_image="https://example.com/file",
            ratio="1280:720",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            image_to_video = await response.parse()
            assert_matches_type(ImageToVideoCreateResponse, image_to_video, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable
from typing_extensions import Literal, overload

import httpx

from ..types import image_to_video_create_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import required_args, maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..lib.polling import (
    NewTaskCreatedResponse,
    AsyncNewTaskCreatedResponse,
    create_waitable_resource,
    create_async_waitable_resource,
)
from .._base_client import make_request_options
from ..types.image_to_video_create_response import ImageToVideoCreateResponse

__all__ = ["ImageToVideoResource", "AsyncImageToVideoResource"]


class ImageToVideoResource(SyncAPIResource):
    """These endpoints all kick off tasks to create generations."""

    @cached_property
    def with_raw_response(self) -> ImageToVideoResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/runwayml/sdk-python#accessing-raw-response-data-eg-headers
        """
        return ImageToVideoResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ImageToVideoResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/runwayml/sdk-python#with_streaming_response
        """
        return ImageToVideoResourceWithStreamingResponse(self)

    @overload
    def create(
        self,
        *,
        duration: int,
        model: Literal["gen4.5"],
        prompt_image: Union[str, Iterable[image_to_video_create_params.Gen4_5PromptImagePromptImage]],
        prompt_text: str,
        ratio: Literal["1280:720", "720:1280", "1104:832", "960:960", "832:1104", "1584:672"],
        content_moderation: image_to_video_create_params.Gen4_5ContentModeration | Omit = omit,
        seed: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> NewTaskCreatedResponse:
        """
        This endpoint will start a new task to generate a video from an image.

        Args:
          duration: The number of seconds of duration for the output video. Must be an integer from
              2 to 10.

          prompt_image: A HTTPS URL.

          prompt_text: A non-empty string up to 1000 characters (measured in UTF-16 code units). This
              should describe in detail what should appear in the output.

          ratio: The resolution of the output video.

          content_moderation: Settings that affect the behavior of the content moderation system.

          seed: If unspecified, a random number is chosen. Varying the seed integer is a way to
              get different results for the same other request parameters. Using the same seed
              integer for an identical request will produce similar results.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def create(
        self,
        *,
        model: Literal["gen4_turbo"],
        prompt_image: Union[str, Iterable[image_to_video_create_params.Gen4TurboPromptImagePromptImage]],
        ratio: Literal["1280:720", "720:1280", "1104:832", "832:1104", "960:960", "1584:672"],
        content_moderation: image_to_video_create_params.Gen4TurboContentModeration | Omit = omit,
        duration: int | Omit = omit,
        prompt_text: str | Omit = omit,
        seed: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> NewTaskCreatedResponse:
        """
        This endpoint will start a new task to generate a video from an image.

        Args:
          prompt_image: A HTTPS URL.

          ratio: The resolution of the output video.

          content_moderation: Settings that affect the behavior of the content moderation system.

          duration: The number of seconds of duration for the output video.

          prompt_text: A non-empty string up to 1000 characters (measured in UTF-16 code units). This
              should describe in detail what should appear in the output.

          seed: If unspecified, a random number is chosen. Varying the seed integer is a way to
              get different results for the same other request parameters. Using the same seed
              integer for an identical request will produce similar results.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def create(
        self,
        *,
        model: Literal["veo3.1"],
        prompt_image: Union[str, Iterable[image_to_video_create_params.Veo3_1PromptImagePromptImage]],
        ratio: Literal["1280:720", "720:1280", "1080:1920", "1920:1080"],
        audio: bool | Omit = omit,
        duration: Literal[4, 6, 8] | Omit = omit,
        prompt_text: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> NewTaskCreatedResponse:
        """
        This endpoint will start a new task to generate a video from an image.

        Args:
          prompt_image: You may specify an image to use as the first frame of the output video, or an
              array with a first frame and optionally a last frame. This model does not
              support generating with only a last frame.

          ratio: The resolution of the output video.

          audio: Whether to generate audio for the video. Audio inclusion affects pricing.

          duration: The number of seconds of duration for the output video.

          prompt_text: A non-empty string up to 1000 characters (measured in UTF-16 code units). This
              should describe in detail what should appear in the output.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def create(
        self,
        *,
        model: Literal["gen3a_turbo"],
        prompt_image: Union[str, Iterable[image_to_video_create_params.Gen3aTurboPromptImagePromptImage]],
        prompt_text: str,
        content_moderation: image_to_video_create_params.Gen3aTurboContentModeration | Omit = omit,
        duration: Literal[5, 10] | Omit = omit,
        ratio: Literal["768:1280", "1280:768"] | Omit = omit,
        seed: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> NewTaskCreatedResponse:
        """
        This endpoint will start a new task to generate a video from an image.

        Args:
          prompt_image: A HTTPS URL.

          prompt_text: A non-empty string up to 1000 characters (measured in UTF-16 code units). This
              should describe in detail what should appear in the output.

          content_moderation: Settings that affect the behavior of the content moderation system.

          duration: The duration of the output video in seconds.

          ratio: The resolution of the output video.

          seed: If unspecified, a random number is chosen. Varying the seed integer is a way to
              get different results for the same other request parameters. Using the same seed
              integer for an identical request will produce similar results.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def create(
        self,
        *,
        model: Literal["veo3.1_fast"],
        prompt_image: Union[str, Iterable[image_to_video_create_params.Veo3_1FastPromptImagePromptImage]],
        ratio: Literal["1280:720", "720:1280", "1080:1920", "1920:1080"],
        audio: bool | Omit = omit,
        duration: Literal[4, 6, 8] | Omit = omit,
        prompt_text: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> NewTaskCreatedResponse:
        """
        This endpoint will start a new task to generate a video from an image.

        Args:
          prompt_image: You may specify an image to use as the first frame of the output video, or an
              array with a first frame and optionally a last frame. This model does not
              support generating with only a last frame.

          ratio: The resolution of the output video.

          audio: Whether to generate audio for the video. Audio inclusion affects pricing.

          duration: The number of seconds of duration for the output video.

          prompt_text: A non-empty string up to 1000 characters (measured in UTF-16 code units). This
              should describe in detail what should appear in the output.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def create(
        self,
        *,
        duration: Literal[8],
        model: Literal["veo3"],
        prompt_image: Union[str, Iterable[image_to_video_create_params.Veo3PromptImagePromptImage]],
        ratio: Literal["1280:720", "720:1280", "1080:1920", "1920:1080"],
        prompt_text: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> NewTaskCreatedResponse:
        """
        This endpoint will start a new task to generate a video from an image.

        Args:
          duration: The number of seconds of duration for the output video.

          prompt_image: A HTTPS URL.

          ratio: The resolution of the output video.

          prompt_text: A non-empty string up to 1000 characters (measured in UTF-16 code units). This
              should describe in detail what should appear in the output.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(
        ["duration", "model", "prompt_image", "prompt_text", "ratio"],
        ["model", "prompt_image", "ratio"],
        ["model", "prompt_image", "prompt_text"],
        ["duration", "model", "prompt_image", "ratio"],
    )
    def create(
        self,
        *,
        duration: int | Literal[4, 6, 8] | Literal[5, 10] | Literal[8] | Omit = omit,
        model: Literal["gen4.5"]
        | Literal["gen4_turbo"]
        | Literal["veo3.1"]
        | Literal["gen3a_turbo"]
        | Literal["veo3.1_fast"]
        | Literal["veo3"],
        prompt_image: Union[str, Iterable[image_to_video_create_params.Gen4_5PromptImagePromptImage]]
        | Union[str, Iterable[image_to_video_create_params.Gen4TurboPromptImagePromptImage]]
        | Union[str, Iterable[image_to_video_create_params.Gen3aTurboPromptImagePromptImage]]
        | Union[str, Iterable[image_to_video_create_params.Veo3_1PromptImagePromptImage]]
        | Union[str, Iterable[image_to_video_create_params.Veo3_1FastPromptImagePromptImage]]
        | Union[str, Iterable[image_to_video_create_params.Veo3PromptImagePromptImage]],
        prompt_text: str | Omit = omit,
        ratio: Literal["1280:720", "720:1280", "1104:832", "960:960", "832:1104", "1584:672"]
        | Literal["1280:720", "720:1280", "1104:832", "832:1104", "960:960", "1584:672"]
        | Literal["1280:720", "720:1280", "1080:1920", "1920:1080"]
        | Literal["768:1280", "1280:768"]
        | Omit = omit,
        content_moderation: image_to_video_create_params.Gen4_5ContentModeration
        | image_to_video_create_params.Gen4TurboContentModeration
        | image_to_video_create_params.Gen3aTurboContentModeration
        | Omit = omit,
        seed: int | Omit = omit,
        audio: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> NewTaskCreatedResponse:
        return self._post(
            "/v1/image_to_video",
            body=maybe_transform(
                {
                    "duration": duration,
                    "model": model,
                    "prompt_image": prompt_image,
                    "prompt_text": prompt_text,
                    "ratio": ratio,
                    "content_moderation": content_moderation,
                    "seed": seed,
                    "audio": audio,
                },
                image_to_video_create_params.ImageToVideoCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=create_waitable_resource(ImageToVideoCreateResponse, self._client),
        )


class AsyncImageToVideoResource(AsyncAPIResource):
    """These endpoints all kick off tasks to create generations."""

    @cached_property
    def with_raw_response(self) -> AsyncImageToVideoResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/runwayml/sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncImageToVideoResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncImageToVideoResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/runwayml/sdk-python#with_streaming_response
        """
        return AsyncImageToVideoResourceWithStreamingResponse(self)

    @overload
    async def create(
        self,
        *,
        duration: int,
        model: Literal["gen4.5"],
        prompt_image: Union[str, Iterable[image_to_video_create_params.Gen4_5PromptImagePromptImage]],
        prompt_text: str,
        ratio: Literal["1280:720", "720:1280", "1104:832", "960:960", "832:1104", "1584:672"],
        content_moderation: image_to_video_create_params.Gen4_5ContentModeration | Omit = omit,
        seed: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncNewTaskCreatedResponse:
        """
        This endpoint will start a new task to generate a video from an image.

        Args:
          duration: The number of seconds of duration for the output video. Must be an integer from
              2 to 10.

          prompt_image: A HTTPS URL.

          prompt_text: A non-empty string up to 1000 characters (measured in UTF-16 code units). This
              should describe in detail what should appear in the output.

          ratio: The resolution of the output video.

          content_moderation: Settings that affect the behavior of the content moderation system.

          seed: If unspecified, a random number is chosen. Varying the seed integer is a way to
              get different results for the same other request parameters. Using the same seed
              integer for an identical request will produce similar results.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def create(
        self,
        *,
        model: Literal["gen4_turbo"],
        prompt_image: Union[str, Iterable[image_to_video_create_params.Gen4TurboPromptImagePromptImage]],
        ratio: Literal["1280:720", "720:1280", "1104:832", "832:1104", "960:960", "1584:672"],
        content_moderation: image_to_video_create_params.Gen4TurboContentModeration | Omit = omit,
        duration: int | Omit = omit,
        prompt_text: str | Omit = omit,
        seed: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncNewTaskCreatedResponse:
        """
        This endpoint will start a new task to generate a video from an image.

        Args:
          prompt_image: A HTTPS URL.

          ratio: The resolution of the output video.

          content_moderation: Settings that affect the behavior of the content moderation system.

          duration: The number of seconds of duration for the output video.

          prompt_text: A non-empty string up to 1000 characters (measured in UTF-16 code units). This
              should describe in detail what should appear in the output.

          seed: If unspecified, a random number is chosen. Varying the seed integer is a way to
              get different results for the same other request parameters. Using the same seed
              integer for an identical request will produce similar results.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def create(
        self,
        *,
        model: Literal["veo3.1"],
        prompt_image: Union[str, Iterable[image_to_video_create_params.Veo3_1PromptImagePromptImage]],
        ratio: Literal["1280:720", "720:1280", "1080:1920", "1920:1080"],
        audio: bool | Omit = omit,
        duration: Literal[4, 6, 8] | Omit = omit,
        prompt_text: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncNewTaskCreatedResponse:
        """
        This endpoint will start a new task to generate a video from an image.

        Args:
          prompt_image: You may specify an image to use as the first frame of the output video, or an
              array with a first frame and optionally a last frame. This model does not
              support generating with only a last frame.

          ratio: The resolution of the output video.

          audio: Whether to generate audio for the video. Audio inclusion affects pricing.

          duration: The number of seconds of duration for the output video.

          prompt_text: A non-empty string up to 1000 characters (measured in UTF-16 code units). This
              should describe in detail what should appear in the output.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def create(
        self,
        *,
        model: Literal["gen3a_turbo"],
        prompt_image: Union[str, Iterable[image_to_video_create_params.Gen3aTurboPromptImagePromptImage]],
        prompt_text: str,
        content_moderation: image_to_video_create_params.Gen3aTurboContentModeration | Omit = omit,
        duration: Literal[5, 10] | Omit = omit,
        ratio: Literal["768:1280", "1280:768"] | Omit = omit,
        seed: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncNewTaskCreatedResponse:
        """
        This endpoint will start a new task to generate a video from an image.

        Args:
          prompt_image: A HTTPS URL.

          prompt_text: A non-empty string up to 1000 characters (measured in UTF-16 code units). This
              should describe in detail what should appear in the output.

          content_moderation: Settings that affect the behavior of the content moderation system.

          duration: The duration of the output video in seconds.

          ratio: The resolution of the output video.

          seed: If unspecified, a random number is chosen. Varying the seed integer is a way to
              get different results for the same other request parameters. Using the same seed
              integer for an identical request will produce similar results.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def create(
        self,
        *,
        model: Literal["veo3.1_fast"],
        prompt_image: Union[str, Iterable[image_to_video_create_params.Veo3_1FastPromptImagePromptImage]],
        ratio: Literal["1280:720", "720:1280", "1080:1920", "1920:1080"],
        audio: bool | Omit = omit,
        duration: Literal[4, 6, 8] | Omit = omit,
        prompt_text: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncNewTaskCreatedResponse:
        """
        This endpoint will start a new task to generate a video from an image.

        Args:
          prompt_image: You may specify an image to use as the first frame of the output video, or an
              array with a first frame and optionally a last frame. This model does not
              support generating with only a last frame.

          ratio: The resolution of the output video.

          audio: Whether to generate audio for the video. Audio inclusion affects pricing.

          duration: The number of seconds of duration for the output video.

          prompt_text: A non-empty string up to 1000 characters (measured in UTF-16 code units). This
              should describe in detail what should appear in the output.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def create(
        self,
        *,
        duration: Literal[8],
        model: Literal["veo3"],
        prompt_image: Union[str, Iterable[image_to_video_create_params.Veo3PromptImagePromptImage]],
        ratio: Literal["1280:720", "720:1280", "1080:1920", "1920:1080"],
        prompt_text: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncNewTaskCreatedResponse:
        """
        This endpoint will start a new task to generate a video from an image.

        Args:
          duration: The number of seconds of duration for the output video.

          prompt_image: A HTTPS URL.

          ratio: The resolution of the output video.

          prompt_text: A non-empty string up to 1000 characters (measured in UTF-16 code units). This
              should describe in detail what should appear in the output.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(
        ["duration", "model", "prompt_image", "prompt_text", "ratio"],
        ["model", "prompt_image", "ratio"],
        ["model", "prompt_image", "prompt_text"],
        ["duration", "model", "prompt_image", "ratio"],
    )
    async def create(
        self,
        *,
        duration: int | Literal[4, 6, 8] | Literal[5, 10] | Literal[8] | Omit = omit,
        model: Literal["gen4.5"]
        | Literal["gen4_turbo"]
        | Literal["veo3.1"]
        | Literal["gen3a_turbo"]
        | Literal["veo3.1_fast"]
        | Literal["veo3"],
        prompt_image: Union[str, Iterable[image_to_video_create_params.Gen4_5PromptImagePromptImage]]
        | Union[str, Iterable[image_to_video_create_params.Gen4TurboPromptImagePromptImage]]
        | Union[str, Iterable[image_to_video_create_params.Gen3aTurboPromptImagePromptImage]]
        | Union[str, Iterable[image_to_video_create_params.Veo3_1PromptImagePromptImage]]
        | Union[str, Iterable[image_to_video_create_params.Veo3_1FastPromptImagePromptImage]]
        | Union[str, Iterable[image_to_video_create_params.Veo3PromptImagePromptImage]],
        prompt_text: str | Omit = omit,
        ratio: Literal["1280:720", "720:1280", "1104:832", "960:960", "832:1104", "1584:672"]
        | Literal["1280:720", "720:1280", "1104:832", "832:1104", "960:960", "1584:672"]
        | Literal["1280:720", "720:1280", "1080:1920", "1920:1080"]
        | Literal["768:1280", "1280:768"]
        | Omit = omit,
        content_moderation: image_to_video_create_params.Gen4_5ContentModeration
        | image_to_video_create_params.Gen4TurboContentModeration
        | image_to_video_create_params.Gen3aTurboContentModeration
        | Omit = omit,
        seed: int | Omit = omit,
        audio: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncNewTaskCreatedResponse:
        return await self._post(
            "/v1/image_to_video",
            body=await async_maybe_transform(
                {
                    "duration": duration,
                    "model": model,
                    "prompt_image": prompt_image,
                    "prompt_text": prompt_text,
                    "ratio": ratio,
                    "content_moderation": content_moderation,
                    "seed": seed,
                    "audio": audio,
                },
                image_to_video_create_params.ImageToVideoCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=create_async_waitable_resource(ImageToVideoCreateResponse, self._client),
        )


class ImageToVideoResourceWithRawResponse:
    def __init__(self, image_to_video: ImageToVideoResource) -> None:
        self._image_to_video = image_to_video

        self.create = to_raw_response_wrapper(
            image_to_video.create,
        )


class AsyncImageToVideoResourceWithRawResponse:
    def __init__(self, image_to_video: AsyncImageToVideoResource) -> None:
        self._image_to_video = image_to_video

        self.create = async_to_raw_response_wrapper(
            image_to_video.create,
        )


class ImageToVideoResourceWithStreamingResponse:
    def __init__(self, image_to_video: ImageToVideoResource) -> None:
        self._image_to_video = image_to_video

        self.create = to_streamed_response_wrapper(
            image_to_video.create,
        )


class AsyncImageToVideoResourceWithStreamingResponse:
    def __init__(self, image_to_video: AsyncImageToVideoResource) -> None:
        self._image_to_video = image_to_video

        self.create = async_to_streamed_response_wrapper(
            image_to_video.create,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import io
import os
from typing import Any
from pathlib import Path

import httpx
import pytest
from respx import MockRouter

from runwayml import RunwayML, AsyncRunwayML

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")
sample_file = Path(__file__).parent.parent / "sample_file.txt"


class TestUploads:
    @pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])
    @pytest.mark.parametrize(
        "file",
        [
            b"raw bytes content",
            io.BytesIO(b"content"),
            (None, b"content"),
        ],
        ids=["raw_bytes", "bytesio_no_name", "tuple_none_filename"],
    )
    def test_create_ephemeral_rejects_invalid_files(self, client: RunwayML, file: Any) -> None:
        """Files without inferrable filename should be rejected with clear error."""
        with pytest.raises(ValueError, match="Cannot infer filename from file"):
            client.uploads.create_ephemeral(file=file)

    @pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])
    @pytest.mark.respx
    @pytest.mark.parametrize(
        "file_type",
        ["path", "tuple", "file_object"],
        ids=["path", "tuple_with_filename", "file_object_with_name"],
    )
    def test_create_ephemeral_accepts_valid_files(
        self, respx_mock: MockRouter, client: RunwayML, file_type: str
    ) -> None:
        """Files with inferrable filename should be accepted."""
        if file_type == "path":
            file: Any = sample_file
        elif file_type == "tuple":
            file = ("test.txt", b"content")
        else:  # file_object
            file = open(sample_file, "rb")

        respx_mock.post(f"{base_url}/v1/uploads").mock(
            return_value=httpx.Response(
                200,
                json={
                    "runwayUri": "runway://upload/test123",
                    "uploadUrl": "https://storage.example.com/upload",
                    "fields": {"key": "value"},
                },
            )
        )
        respx_mock.post("https://storage.example.com/upload").mock(return_value=httpx.Response(204))

        try:
            response = client.uploads.create_ephemeral(file=file)
            assert response.uri == "runway://upload/test123"
        finally:
            if file_type == "file_object":
                file.close()


class TestAsyncUploads:
    @pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])
    @pytest.mark.parametrize(
        "file",
        [
            b"raw bytes content",
            io.BytesIO(b"content"),
            (None, b"content"),
        ],
        ids=["raw_bytes", "bytesio_no_name", "tuple_none_filename"],
    )
    async def test_create_ephemeral_rejects_invalid_files(self, async_client: AsyncRunwayML, file: Any) -> None:
        """Files without inferrable filename should be rejected with clear error."""
        with pytest.raises(ValueError, match="Cannot infer filename from file"):
            await async_client.uploads.create_ephemeral(file=file)

    @pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])
    @pytest.mark.respx
    @pytest.mark.parametrize(
        "file_type",
        ["path", "tuple", "file_object"],
        ids=["path", "tuple_with_filename", "file_object_with_name"],
    )
    async def test_create_ephemeral_accepts_valid_files(
        self, respx_mock: MockRouter, async_client: AsyncRunwayML, file_type: str
    ) -> None:
        """Files with inferrable filename should be accepted."""
        if file_type == "path":
            file: Any = sample_file
        elif file_type == "tuple":
            file = ("test.txt", b"content")
        else:  # file_object
            file = open(sample_file, "rb")

        respx_mock.post(f"{base_url}/v1/uploads").mock(
            return_value=httpx.Response(
                200,
                json={
                    "runwayUri": "runway://upload/test123",
                    "uploadUrl": "https://storage.example.com/upload",
                    "fields": {"key": "value"},
                },
            )
        )
        respx_mock.post("https://storage.example.com/upload").mock(return_value=httpx.Response(204))

        try:
            response = await async_client.uploads.create_ephemeral(file=file)
            assert response.uri == "runway://upload/test123"
        finally:
            if file_type == "file_object":
                file.close()



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Literal, overload

import httpx

from ..types import text_to_image_create_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import required_args, maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..lib.polling import (
    NewTaskCreatedResponse,
    AsyncNewTaskCreatedResponse,
    create_waitable_resource,
    create_async_waitable_resource,
)
from .._base_client import make_request_options
from ..types.text_to_image_create_response import TextToImageCreateResponse

__all__ = ["TextToImageResource", "AsyncTextToImageResource"]


class TextToImageResource(SyncAPIResource):
    """These endpoints all kick off tasks to create generations."""

    @cached_property
    def with_raw_response(self) -> TextToImageResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/runwayml/sdk-python#accessing-raw-response-data-eg-headers
        """
        return TextToImageResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TextToImageResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/runwayml/sdk-python#with_streaming_response
        """
        return TextToImageResourceWithStreamingResponse(self)

    @overload
    def create(
        self,
        *,
        model: Literal["gen4_image_turbo"],
        prompt_text: str,
        ratio: Literal[
            "1024:1024",
            "1080:1080",
            "1168:880",
            "1360:768",
            "1440:1080",
            "1080:1440",
            "1808:768",
            "1920:1080",
            "1080:1920",
            "2112:912",
            "1280:720",
            "720:1280",
            "720:720",
            "960:720",
            "720:960",
            "1680:720",
        ],
        reference_images: Iterable[text_to_image_create_params.Gen4ImageTurboReferenceImage],
        content_moderation: text_to_image_create_params.Gen4ImageTurboContentModeration | Omit = omit,
        seed: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> NewTaskCreatedResponse:
        """
        This endpoint will start a new task to generate images from text and/or image(s)

        Args:
          prompt_text: A non-empty string up to 1000 characters (measured in UTF-16 code units). This
              should describe in detail what should appear in the output.

          ratio: The resolution of the output image.

          reference_images: An array of one to three images to be used as references for the generated image
              output.

          content_moderation: Settings that affect the behavior of the content moderation system.

          seed: If unspecified, a random number is chosen. Varying the seed integer is a way to
              get different results for the same other request parameters. Using the same seed
              integer for an identical request will produce similar results.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def create(
        self,
        *,
        model: Literal["gen4_image"],
        prompt_text: str,
        ratio: Literal[
            "1024:1024",
            "1080:1080",
            "1168:880",
            "1360:768",
            "1440:1080",
            "1080:1440",
            "1808:768",
            "1920:1080",
            "1080:1920",
            "2112:912",
            "1280:720",
            "720:1280",
            "720:720",
            "960:720",
            "720:960",
            "1680:720",
        ],
        content_moderation: text_to_image_create_params.Gen4ImageContentModeration | Omit = omit,
        reference_images: Iterable[text_to_image_create_params.Gen4ImageReferenceImage] | Omit = omit,
        seed: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> NewTaskCreatedResponse:
        """
        This endpoint will start a new task to generate images from text and/or image(s)

        Args:
          prompt_text: A non-empty string up to 1000 characters (measured in UTF-16 code units). This
              should describe in detail what should appear in the output.

          ratio: The resolution of the output image.

          content_moderation: Settings that affect the behavior of the content moderation system.

          reference_images: An array of up to three images to be used as references for the generated image
              output.

          seed: If unspecified, a random number is chosen. Varying the seed integer is a way to
              get different results for the same other request parameters. Using the same seed
              integer for an identical request will produce similar results.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def create(
        self,
        *,
        model: Literal["gemini_2.5_flash"],
        prompt_text: str,
        ratio: Literal[
            "1344:768",
            "768:1344",
            "1024:1024",
            "1184:864",
            "864:1184",
            "1536:672",
            "832:1248",
            "1248:832",
            "896:1152",
            "1152:896",
        ],
        reference_images: Iterable[text_to_image_create_params.Gemini2_5FlashReferenceImage] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> NewTaskCreatedResponse:
        """
        This endpoint will start a new task to generate images from text and/or image(s)

        Args:
          prompt_text: A non-empty string up to 1000 characters (measured in UTF-16 code units). This
              should describe in detail what should appear in the output.

          ratio: The resolution of the output image.

          reference_images: An array of up to three images to be used as references for the generated image
              output.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["model", "prompt_text", "ratio", "reference_images"], ["model", "prompt_text", "ratio"])
    def create(
        self,
        *,
        model: Literal["gen4_image_turbo"] | Literal["gen4_image"] | Literal["gemini_2.5_flash"],
        prompt_text: str,
        ratio: Literal[
            "1024:1024",
            "1080:1080",
            "1168:880",
            "1360:768",
            "1440:1080",
            "1080:1440",
            "1808:768",
            "1920:1080",
            "1080:1920",
            "2112:912",
            "1280:720",
            "720:1280",
            "720:720",
            "960:720",
            "720:960",
            "1680:720",
        ]
        | Literal[
            "1344:768",
            "768:1344",
            "1024:1024",
            "1184:864",
            "864:1184",
            "1536:672",
            "832:1248",
            "1248:832",
            "896:1152",
            "1152:896",
        ],
        reference_images: Iterable[text_to_image_create_params.Gen4ImageTurboReferenceImage]
        | Iterable[text_to_image_create_params.Gen4ImageReferenceImage]
        | Iterable[text_to_image_create_params.Gemini2_5FlashReferenceImage]
        | Omit = omit,
        content_moderation: text_to_image_create_params.Gen4ImageTurboContentModeration
        | text_to_image_create_params.Gen4ImageContentModeration
        | Omit = omit,
        seed: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> NewTaskCreatedResponse:
        return self._post(
            "/v1/text_to_image",
            body=maybe_transform(
                {
                    "model": model,
                    "prompt_text": prompt_text,
                    "ratio": ratio,
                    "reference_images": reference_images,
                    "content_moderation": content_moderation,
                    "seed": seed,
                },
                text_to_image_create_params.TextToImageCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=create_waitable_resource(TextToImageCreateResponse, self._client),
        )


class AsyncTextToImageResource(AsyncAPIResource):
    """These endpoints all kick off tasks to create generations."""

    @cached_property
    def with_raw_response(self) -> AsyncTextToImageResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/runwayml/sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncTextToImageResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTextToImageResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/runwayml/sdk-python#with_streaming_response
        """
        return AsyncTextToImageResourceWithStreamingResponse(self)

    @overload
    async def create(
        self,
        *,
        model: Literal["gen4_image_turbo"],
        prompt_text: str,
        ratio: Literal[
            "1024:1024",
            "1080:1080",
            "1168:880",
            "1360:768",
            "1440:1080",
            "1080:1440",
            "1808:768",
            "1920:1080",
            "1080:1920",
            "2112:912",
            "1280:720",
            "720:1280",
            "720:720",
            "960:720",
            "720:960",
            "1680:720",
        ],
        reference_images: Iterable[text_to_image_create_params.Gen4ImageTurboReferenceImage],
        content_moderation: text_to_image_create_params.Gen4ImageTurboContentModeration | Omit = omit,
        seed: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncNewTaskCreatedResponse:
        """
        This endpoint will start a new task to generate images from text and/or image(s)

        Args:
          prompt_text: A non-empty string up to 1000 characters (measured in UTF-16 code units). This
              should describe in detail what should appear in the output.

          ratio: The resolution of the output image.

          reference_images: An array of one to three images to be used as references for the generated image
              output.

          content_moderation: Settings that affect the behavior of the content moderation system.

          seed: If unspecified, a random number is chosen. Varying the seed integer is a way to
              get different results for the same other request parameters. Using the same seed
              integer for an identical request will produce similar results.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def create(
        self,
        *,
        model: Literal["gen4_image"],
        prompt_text: str,
        ratio: Literal[
            "1024:1024",
            "1080:1080",
            "1168:880",
            "1360:768",
            "1440:1080",
            "1080:1440",
            "1808:768",
            "1920:1080",
            "1080:1920",
            "2112:912",
            "1280:720",
            "720:1280",
            "720:720",
            "960:720",
            "720:960",
            "1680:720",
        ],
        content_moderation: text_to_image_create_params.Gen4ImageContentModeration | Omit = omit,
        reference_images: Iterable[text_to_image_create_params.Gen4ImageReferenceImage] | Omit = omit,
        seed: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncNewTaskCreatedResponse:
        """
        This endpoint will start a new task to generate images from text and/or image(s)

        Args:
          prompt_text: A non-empty string up to 1000 characters (measured in UTF-16 code units). This
              should describe in detail what should appear in the output.

          ratio: The resolution of the output image.

          content_moderation: Settings that affect the behavior of the content moderation system.

          reference_images: An array of up to three images to be used as references for the generated image
              output.

          seed: If unspecified, a random number is chosen. Varying the seed integer is a way to
              get different results for the same other request parameters. Using the same seed
              integer for an identical request will produce similar results.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def create(
        self,
        *,
        model: Literal["gemini_2.5_flash"],
        prompt_text: str,
        ratio: Literal[
            "1344:768",
            "768:1344",
            "1024:1024",
            "1184:864",
            "864:1184",
            "1536:672",
            "832:1248",
            "1248:832",
            "896:1152",
            "1152:896",
        ],
        reference_images: Iterable[text_to_image_create_params.Gemini2_5FlashReferenceImage] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncNewTaskCreatedResponse:
        """
        This endpoint will start a new task to generate images from text and/or image(s)

        Args:
          prompt_text: A non-empty string up to 1000 characters (measured in UTF-16 code units). This
              should describe in detail what should appear in the output.

          ratio: The resolution of the output image.

          reference_images: An array of up to three images to be used as references for the generated image
              output.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["model", "prompt_text", "ratio", "reference_images"], ["model", "prompt_text", "ratio"])
    async def create(
        self,
        *,
        model: Literal["gen4_image_turbo"] | Literal["gen4_image"] | Literal["gemini_2.5_flash"],
        prompt_text: str,
        ratio: Literal[
            "1024:1024",
            "1080:1080",
            "1168:880",
            "1360:768",
            "1440:1080",
            "1080:1440",
            "1808:768",
            "1920:1080",
            "1080:1920",
            "2112:912",
            "1280:720",
            "720:1280",
            "720:720",
            "960:720",
            "720:960",
            "1680:720",
        ]
        | Literal[
            "1344:768",
            "768:1344",
            "1024:1024",
            "1184:864",
            "864:1184",
            "1536:672",
            "832:1248",
            "1248:832",
            "896:1152",
            "1152:896",
        ],
        reference_images: Iterable[text_to_image_create_params.Gen4ImageTurboReferenceImage]
        | Iterable[text_to_image_create_params.Gen4ImageReferenceImage]
        | Iterable[text_to_image_create_params.Gemini2_5FlashReferenceImage]
        | Omit = omit,
        content_moderation: text_to_image_create_params.Gen4ImageTurboContentModeration
        | text_to_image_create_params.Gen4ImageContentModeration
        | Omit = omit,
        seed: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncNewTaskCreatedResponse:
        return await self._post(
            "/v1/text_to_image",
            body=await async_maybe_transform(
                {
                    "model": model,
                    "prompt_text": prompt_text,
                    "ratio": ratio,
                    "reference_images": reference_images,
                    "content_moderation": content_moderation,
                    "seed": seed,
                },
                text_to_image_create_params.TextToImageCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=create_async_waitable_resource(TextToImageCreateResponse, self._client),
        )


class TextToImageResourceWithRawResponse:
    def __init__(self, text_to_image: TextToImageResource) -> None:
        self._text_to_image = text_to_image

        self.create = to_raw_response_wrapper(
            text_to_image.create,
        )


class AsyncTextToImageResourceWithRawResponse:
    def __init__(self, text_to_image: AsyncTextToImageResource) -> None:
        self._text_to_image = text_to_image

        self.create = async_to_raw_response_wrapper(
            text_to_image.create,
        )


class TextToImageResourceWithStreamingResponse:
    def __init__(self, text_to_image: TextToImageResource) -> None:
        self._text_to_image = text_to_image

        self.create = to_streamed_response_wrapper(
            text_to_image.create,
        )


class AsyncTextToImageResourceWithStreamingResponse:
    def __init__(self, text_to_image: AsyncTextToImageResource) -> None:
        self._text_to_image = text_to_image

        self.create = async_to_streamed_response_wrapper(
            text_to_image.create,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .tasks import (
    TasksResource,
    AsyncTasksResource,
    TasksResourceWithRawResponse,
    AsyncTasksResourceWithRawResponse,
    TasksResourceWithStreamingResponse,
    AsyncTasksResourceWithStreamingResponse,
)
from .uploads import (
    UploadsResource,
    AsyncUploadsResource,
    UploadsResourceWithRawResponse,
    AsyncUploadsResourceWithRawResponse,
    UploadsResourceWithStreamingResponse,
    AsyncUploadsResourceWithStreamingResponse,
)
from .organization import (
    OrganizationResource,
    AsyncOrganizationResource,
    OrganizationResourceWithRawResponse,
    AsyncOrganizationResourceWithRawResponse,
    OrganizationResourceWithStreamingResponse,
    AsyncOrganizationResourceWithStreamingResponse,
)
from .sound_effect import (
    SoundEffectResource,
    AsyncSoundEffectResource,
    SoundEffectResourceWithRawResponse,
    AsyncSoundEffectResourceWithRawResponse,
    SoundEffectResourceWithStreamingResponse,
    AsyncSoundEffectResourceWithStreamingResponse,
)
from .text_to_image import (
    TextToImageResource,
    AsyncTextToImageResource,
    TextToImageResourceWithRawResponse,
    AsyncTextToImageResourceWithRawResponse,
    TextToImageResourceWithStreamingResponse,
    AsyncTextToImageResourceWithStreamingResponse,
)
from .text_to_video import (
    TextToVideoResource,
    AsyncTextToVideoResource,
    TextToVideoResourceWithRawResponse,
    AsyncTextToVideoResourceWithRawResponse,
    TextToVideoResourceWithStreamingResponse,
    AsyncTextToVideoResourceWithStreamingResponse,
)
from .voice_dubbing import (
    VoiceDubbingResource,
    AsyncVoiceDubbingResource,
    VoiceDubbingResourceWithRawResponse,
    AsyncVoiceDubbingResourceWithRawResponse,
    VoiceDubbingResourceWithStreamingResponse,
    AsyncVoiceDubbingResourceWithStreamingResponse,
)
from .image_to_video import (
    ImageToVideoResource,
    AsyncImageToVideoResource,
    ImageToVideoResourceWithRawResponse,
    AsyncImageToVideoResourceWithRawResponse,
    ImageToVideoResourceWithStreamingResponse,
    AsyncImageToVideoResourceWithStreamingResponse,
)
from .text_to_speech import (
    TextToSpeechResource,
    AsyncTextToSpeechResource,
    TextToSpeechResourceWithRawResponse,
    AsyncTextToSpeechResourceWithRawResponse,
    TextToSpeechResourceWithStreamingResponse,
    AsyncTextToSpeechResourceWithStreamingResponse,
)
from .video_to_video import (
    VideoToVideoResource,
    AsyncVideoToVideoResource,
    VideoToVideoResourceWithRawResponse,
    AsyncVideoToVideoResourceWithRawResponse,
    VideoToVideoResourceWithStreamingResponse,
    AsyncVideoToVideoResourceWithStreamingResponse,
)
from .voice_isolation import (
    VoiceIsolationResource,
    AsyncVoiceIsolationResource,
    VoiceIsolationResourceWithRawResponse,
    AsyncVoiceIsolationResourceWithRawResponse,
    VoiceIsolationResourceWithStreamingResponse,
    AsyncVoiceIsolationResourceWithStreamingResponse,
)
from .speech_to_speech import (
    SpeechToSpeechResource,
    AsyncSpeechToSpeechResource,
    SpeechToSpeechResourceWithRawResponse,
    AsyncSpeechToSpeechResourceWithRawResponse,
    SpeechToSpeechResourceWithStreamingResponse,
    AsyncSpeechToSpeechResourceWithStreamingResponse,
)
from .character_performance import (
    CharacterPerformanceResource,
    AsyncCharacterPerformanceResource,
    CharacterPerformanceResourceWithRawResponse,
    AsyncCharacterPerformanceResourceWithRawResponse,
    CharacterPerformanceResourceWithStreamingResponse,
    AsyncCharacterPerformanceResourceWithStreamingResponse,
)

__all__ = [
    "TasksResource",
    "AsyncTasksResource",
    "TasksResourceWithRawResponse",
    "AsyncTasksResourceWithRawResponse",
    "TasksResourceWithStreamingResponse",
    "AsyncTasksResourceWithStreamingResponse",
    "ImageToVideoResource",
    "AsyncImageToVideoResource",
    "ImageToVideoResourceWithRawResponse",
    "AsyncImageToVideoResourceWithRawResponse",
    "ImageToVideoResourceWithStreamingResponse",
    "AsyncImageToVideoResourceWithStreamingResponse",
    "VideoToVideoResource",
    "AsyncVideoToVideoResource",
    "VideoToVideoResourceWithRawResponse",
    "AsyncVideoToVideoResourceWithRawResponse",
    "VideoToVideoResourceWithStreamingResponse",
    "AsyncVideoToVideoResourceWithStreamingResponse",
    "TextToVideoResource",
    "AsyncTextToVideoResource",
    "TextToVideoResourceWithRawResponse",
    "AsyncTextToVideoResourceWithRawResponse",
    "TextToVideoResourceWithStreamingResponse",
    "AsyncTextToVideoResourceWithStreamingResponse",
    "TextToImageResource",
    "AsyncTextToImageResource",
    "TextToImageResourceWithRawResponse",
    "AsyncTextToImageResourceWithRawResponse",
    "TextToImageResourceWithStreamingResponse",
    "AsyncTextToImageResourceWithStreamingResponse",
    "CharacterPerformanceResource",
    "AsyncCharacterPerformanceResource",
    "CharacterPerformanceResourceWithRawResponse",
    "AsyncCharacterPerformanceResourceWithRawResponse",
    "CharacterPerformanceResourceWithStreamingResponse",
    "AsyncCharacterPerformanceResourceWithStreamingResponse",
    "TextToSpeechResource",
    "AsyncTextToSpeechResource",
    "TextToSpeechResourceWithRawResponse",
    "AsyncTextToSpeechResourceWithRawResponse",
    "TextToSpeechResourceWithStreamingResponse",
    "AsyncTextToSpeechResourceWithStreamingResponse",
    "SoundEffectResource",
    "AsyncSoundEffectResource",
    "SoundEffectResourceWithRawResponse",
    "AsyncSoundEffectResourceWithRawResponse",
    "SoundEffectResourceWithStreamingResponse",
    "AsyncSoundEffectResourceWithStreamingResponse",
    "VoiceIsolationResource",
    "AsyncVoiceIsolationResource",
    "VoiceIsolationResourceWithRawResponse",
    "AsyncVoiceIsolationResourceWithRawResponse",
    "VoiceIsolationResourceWithStreamingResponse",
    "AsyncVoiceIsolationResourceWithStreamingResponse",
    "VoiceDubbingResource",
    "AsyncVoiceDubbingResource",
    "VoiceDubbingResourceWithRawResponse",
    "AsyncVoiceDubbingResourceWithRawResponse",
    "VoiceDubbingResourceWithStreamingResponse",
    "AsyncVoiceDubbingResourceWithStreamingResponse",
    "SpeechToSpeechResource",
    "AsyncSpeechToSpeechResource",
    "SpeechToSpeechResourceWithRawResponse",
    "AsyncSpeechToSpeechResourceWithRawResponse",
    "SpeechToSpeechResourceWithStreamingResponse",
    "AsyncSpeechToSpeechResourceWithStreamingResponse",
    "OrganizationResource",
    "AsyncOrganizationResource",
    "OrganizationResourceWithRawResponse",
    "AsyncOrganizationResourceWithRawResponse",
    "OrganizationResourceWithStreamingResponse",
    "AsyncOrganizationResourceWithStreamingResponse",
    "UploadsResource",
    "AsyncUploadsResource",
    "UploadsResourceWithRawResponse",
    "AsyncUploadsResourceWithRawResponse",
    "UploadsResourceWithStreamingResponse",
    "AsyncUploadsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable
from typing_extensions import Literal, Required, Annotated, TypeAlias, TypedDict

from .._utils import PropertyInfo

__all__ = [
    "TextToImageCreateParams",
    "Gen4ImageTurbo",
    "Gen4ImageTurboReferenceImage",
    "Gen4ImageTurboContentModeration",
    "Gen4Image",
    "Gen4ImageContentModeration",
    "Gen4ImageReferenceImage",
    "Gemini2_5Flash",
    "Gemini2_5FlashReferenceImage",
]


class Gen4ImageTurbo(TypedDict, total=False):
    model: Required[Literal["gen4_image_turbo"]]

    prompt_text: Required[Annotated[str, PropertyInfo(alias="promptText")]]
    """A non-empty string up to 1000 characters (measured in UTF-16 code units).

    This should describe in detail what should appear in the output.
    """

    ratio: Required[
        Literal[
            "1024:1024",
            "1080:1080",
            "1168:880",
            "1360:768",
            "1440:1080",
            "1080:1440",
            "1808:768",
            "1920:1080",
            "1080:1920",
            "2112:912",
            "1280:720",
            "720:1280",
            "720:720",
            "960:720",
            "720:960",
            "1680:720",
        ]
    ]
    """The resolution of the output image."""

    reference_images: Required[Annotated[Iterable[Gen4ImageTurboReferenceImage], PropertyInfo(alias="referenceImages")]]
    """
    An array of one to three images to be used as references for the generated image
    output.
    """

    content_moderation: Annotated[Gen4ImageTurboContentModeration, PropertyInfo(alias="contentModeration")]
    """Settings that affect the behavior of the content moderation system."""

    seed: int
    """If unspecified, a random number is chosen.

    Varying the seed integer is a way to get different results for the same other
    request parameters. Using the same seed integer for an identical request will
    produce similar results.
    """


class Gen4ImageTurboReferenceImage(TypedDict, total=False):
    uri: Required[str]
    """A HTTPS URL."""

    tag: str
    """A tag to identify the reference image.

    This is used to reference the image in prompt text.
    """


class Gen4ImageTurboContentModeration(TypedDict, total=False):
    """Settings that affect the behavior of the content moderation system."""

    public_figure_threshold: Annotated[Literal["auto", "low"], PropertyInfo(alias="publicFigureThreshold")]
    """
    When set to `low`, the content moderation system will be less strict about
    preventing generations that include recognizable public figures.
    """


class Gen4Image(TypedDict, total=False):
    model: Required[Literal["gen4_image"]]

    prompt_text: Required[Annotated[str, PropertyInfo(alias="promptText")]]
    """A non-empty string up to 1000 characters (measured in UTF-16 code units).

    This should describe in detail what should appear in the output.
    """

    ratio: Required[
        Literal[
            "1024:1024",
            "1080:1080",
            "1168:880",
            "1360:768",
            "1440:1080",
            "1080:1440",
            "1808:768",
            "1920:1080",
            "1080:1920",
            "2112:912",
            "1280:720",
            "720:1280",
            "720:720",
            "960:720",
            "720:960",
            "1680:720",
        ]
    ]
    """The resolution of the output image."""

    content_moderation: Annotated[Gen4ImageContentModeration, PropertyInfo(alias="contentModeration")]
    """Settings that affect the behavior of the content moderation system."""

    reference_images: Annotated[Iterable[Gen4ImageReferenceImage], PropertyInfo(alias="referenceImages")]
    """
    An array of up to three images to be used as references for the generated image
    output.
    """

    seed: int
    """If unspecified, a random number is chosen.

    Varying the seed integer is a way to get different results for the same other
    request parameters. Using the same seed integer for an identical request will
    produce similar results.
    """


class Gen4ImageContentModeration(TypedDict, total=False):
    """Settings that affect the behavior of the content moderation system."""

    public_figure_threshold: Annotated[Literal["auto", "low"], PropertyInfo(alias="publicFigureThreshold")]
    """
    When set to `low`, the content moderation system will be less strict about
    preventing generations that include recognizable public figures.
    """


class Gen4ImageReferenceImage(TypedDict, total=False):
    uri: Required[str]
    """A HTTPS URL."""

    tag: str
    """A tag to identify the reference image.

    This is used to reference the image in prompt text.
    """


class Gemini2_5Flash(TypedDict, total=False):
    model: Required[Literal["gemini_2.5_flash"]]

    prompt_text: Required[Annotated[str, PropertyInfo(alias="promptText")]]
    """A non-empty string up to 1000 characters (measured in UTF-16 code units).

    This should describe in detail what should appear in the output.
    """

    ratio: Required[
        Literal[
            "1344:768",
            "768:1344",
            "1024:1024",
            "1184:864",
            "864:1184",
            "1536:672",
            "832:1248",
            "1248:832",
            "896:1152",
            "1152:896",
        ]
    ]
    """The resolution of the output image."""

    reference_images: Annotated[Iterable[Gemini2_5FlashReferenceImage], PropertyInfo(alias="referenceImages")]
    """
    An array of up to three images to be used as references for the generated image
    output.
    """


class Gemini2_5FlashReferenceImage(TypedDict, total=False):
    uri: Required[str]
    """A HTTPS URL."""

    tag: str
    """A tag to identify the reference image.

    This is used to reference the image in prompt text.
    """


TextToImageCreateParams: TypeAlias = Union[Gen4ImageTurbo, Gen4Image, Gemini2_5Flash]



import asyncio
import argparse
from pathlib import Path

from runwayml import RunwayML, AsyncRunwayML

parser = argparse.ArgumentParser(description="Upload a file to RunwayML")
parser.add_argument("--file", required=True, type=Path, help="Path to the file to upload")
parser.add_argument("-i", action="store_true", help="Generate an image with the uploaded file")
parser.add_argument("--async", action="store_true", dest="use_async", help="Use async client")
args = parser.parse_args()

file_path = args.file

if not file_path.exists():
    parser.error(f"File not found: {file_path}")


async def async_upload() -> None:
    client = AsyncRunwayML()
    upload_response = await client.uploads.create_ephemeral(file=file_path)
    print(f"Upload successful! URI: {upload_response.uri}")

    if args.i:
        image_task = await client.text_to_image.create(
            model="gen4_image",
            prompt_text="add a bunny",
            ratio="1920:1080",
            reference_images=[{"uri": upload_response.uri}],
        )
        print(await image_task.wait_for_task_output())


def sync_upload() -> None:
    client = RunwayML()
    upload_response = client.uploads.create_ephemeral(file=file_path)
    print(f"Upload successful! URI: {upload_response.uri}")

    if args.i:
        image_task = client.text_to_image.create(
            model="gen4_image",
            prompt_text="add a bunny",
            ratio="1920:1080",
            reference_images=[{"uri": upload_response.uri}],
        )
        print(image_task.wait_for_task_output())


if args.use_async:
    asyncio.run(async_upload())
else:
    sync_upload()



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Literal

import httpx

from ..types import video_to_video_create_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..lib.polling import (
    NewTaskCreatedResponse,
    AsyncNewTaskCreatedResponse,
    create_waitable_resource,
    create_async_waitable_resource,
)
from .._base_client import make_request_options
from ..types.video_to_video_create_response import VideoToVideoCreateResponse

__all__ = ["VideoToVideoResource", "AsyncVideoToVideoResource"]


class VideoToVideoResource(SyncAPIResource):
    """These endpoints all kick off tasks to create generations."""

    @cached_property
    def with_raw_response(self) -> VideoToVideoResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/runwayml/sdk-python#accessing-raw-response-data-eg-headers
        """
        return VideoToVideoResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> VideoToVideoResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/runwayml/sdk-python#with_streaming_response
        """
        return VideoToVideoResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        model: Literal["gen4_aleph"],
        prompt_text: str,
        video_uri: str,
        content_moderation: video_to_video_create_params.ContentModeration | Omit = omit,
        ratio: Literal["1280:720", "720:1280", "1104:832", "960:960", "832:1104", "1584:672", "848:480", "640:480"]
        | Omit = omit,
        references: Iterable[video_to_video_create_params.Reference] | Omit = omit,
        seed: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> NewTaskCreatedResponse:
        """
        This endpoint will start a new task to generate a video from a video.

        Args:
          prompt_text: A non-empty string up to 1000 characters (measured in UTF-16 code units). This
              should describe in detail what should appear in the output.

          video_uri: A HTTPS URL.

          content_moderation: Settings that affect the behavior of the content moderation system.

          ratio: Deprecated. This field is ignored. The resolution of the output video is
              determined by the input video.

          references: An array of references. Currently up to one reference is supported. See
              [our docs](/assets/inputs#images) on image inputs for more information.

          seed: If unspecified, a random number is chosen. Varying the seed integer is a way to
              get different results for the same other request parameters. Using the same seed
              integer for an identical request will produce similar results.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/video_to_video",
            body=maybe_transform(
                {
                    "model": model,
                    "prompt_text": prompt_text,
                    "video_uri": video_uri,
                    "content_moderation": content_moderation,
                    "ratio": ratio,
                    "references": references,
                    "seed": seed,
                },
                video_to_video_create_params.VideoToVideoCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=create_waitable_resource(VideoToVideoCreateResponse, self._client),
        )


class AsyncVideoToVideoResource(AsyncAPIResource):
    """These endpoints all kick off tasks to create generations."""

    @cached_property
    def with_raw_response(self) -> AsyncVideoToVideoResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/runwayml/sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncVideoToVideoResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncVideoToVideoResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/runwayml/sdk-python#with_streaming_response
        """
        return AsyncVideoToVideoResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        model: Literal["gen4_aleph"],
        prompt_text: str,
        video_uri: str,
        content_moderation: video_to_video_create_params.ContentModeration | Omit = omit,
        ratio: Literal["1280:720", "720:1280", "1104:832", "960:960", "832:1104", "1584:672", "848:480", "640:480"]
        | Omit = omit,
        references: Iterable[video_to_video_create_params.Reference] | Omit = omit,
        seed: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncNewTaskCreatedResponse:
        """
        This endpoint will start a new task to generate a video from a video.

        Args:
          prompt_text: A non-empty string up to 1000 characters (measured in UTF-16 code units). This
              should describe in detail what should appear in the output.

          video_uri: A HTTPS URL.

          content_moderation: Settings that affect the behavior of the content moderation system.

          ratio: Deprecated. This field is ignored. The resolution of the output video is
              determined by the input video.

          references: An array of references. Currently up to one reference is supported. See
              [our docs](/assets/inputs#images) on image inputs for more information.

          seed: If unspecified, a random number is chosen. Varying the seed integer is a way to
              get different results for the same other request parameters. Using the same seed
              integer for an identical request will produce similar results.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/video_to_video",
            body=await async_maybe_transform(
                {
                    "model": model,
                    "prompt_text": prompt_text,
                    "video_uri": video_uri,
                    "content_moderation": content_moderation,
                    "ratio": ratio,
                    "references": references,
                    "seed": seed,
                },
                video_to_video_create_params.VideoToVideoCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=create_async_waitable_resource(VideoToVideoCreateResponse, self._client),
        )


class VideoToVideoResourceWithRawResponse:
    def __init__(self, video_to_video: VideoToVideoResource) -> None:
        self._video_to_video = video_to_video

        self.create = to_raw_response_wrapper(
            video_to_video.create,
        )


class AsyncVideoToVideoResourceWithRawResponse:
    def __init__(self, video_to_video: AsyncVideoToVideoResource) -> None:
        self._video_to_video = video_to_video

        self.create = async_to_raw_response_wrapper(
            video_to_video.create,
        )


class VideoToVideoResourceWithStreamingResponse:
    def __init__(self, video_to_video: VideoToVideoResource) -> None:
        self._video_to_video = video_to_video

        self.create = to_streamed_response_wrapper(
            video_to_video.create,
        )


class AsyncVideoToVideoResourceWithStreamingResponse:
    def __init__(self, video_to_video: AsyncVideoToVideoResource) -> None:
        self._video_to_video = video_to_video

        self.create = async_to_streamed_response_wrapper(
            video_to_video.create,
        )
